/* *****************************************************************************
 * File:        main.c
 * Project:     Mini Anzeige Modul
 * Author:      Nicolas Meyertns
 * Version:     0.1.a.xc8
 * Web:         www.PIC-Projekte.de
 * ****************************************************************************/

#include <xc.h>
#include "dogl.h"
#include "main.h"
#include "menu.h"
#include "font.h"
#include "setup.h"

/*Konfigurationswort*/
#pragma config FOSC = INTIO67   // Internal oscillator block)
#pragma config PWRTEN = ON      // Power up timer enabled
#pragma config BOREN = OFF      // Brown-out Reset disabled
#pragma config LVP = OFF        // Single-Supply ICSP disabled
#pragma config MCLRE = EXTMCLR  // MCLR pin enabled, RE3 input pin disabled
#pragma config WDTEN = OFF      // Watch dog timer is always disabled
#pragma config PBADEN = OFF     // PORTB<5:0> are digital I/O on Reset

uint8_t input = NO_KEY;         // Maskiertes Byte fr Eingabezustnde
uint8_t lastKeyDown = NO_KEY;   // Letzter Zustand
uint8_t encState = 0x00;        // Zustand des Drehgebers (Default V00)

void main(void)
{
    uint8_t actContrast = DEFAULT_CONTRAST;

    /*Datenstruktur fr den Menbaum erstellen*/
    struct sMenuEntry meControll[] =
    {
        { "Energieoptionen", mEnergy },
        { "Informationen", mInfo },
        { "Exit" , 0}
    };

    initPIC();
    configPWM();
    initSPI();
    initDOGL(&actContrast);

    charChainDOGL(0, 0, NOINVERSE, "Warte auf Taste");

    while(1)
    {
        if(input & KEY_OK)
        {
            input &= ~KEY_OK;
            mControll(ARRAY_SIZE(meControll),meControll);
            clearDOGL();
            charChainDOGL(0, 0, NOINVERSE, "Warte auf Taste");
        }
    }
}

void initPIC (void)
{
    /*IO Konfiguration*/
    LATA = 0x00;
    LATB = 0x00;
    LATC = 0x00;
    LATD = 0x00;
    
    /*Display Reset*/
    RST = 0;
    
    TRISA = 0x00;
    TRISB = 0x00;
    TRISC = 0x11;
    TRISD = 0x0C;

    /*PIC spezifisches*/
    OSCCON2bits.MFIOSEL = 0;
    OSCCONbits.IRCF0 = 1;
    OSCCONbits.IRCF1 = 0;
    OSCCONbits.IRCF2 = 1;

    /*Auf digital umschalten*/
    ANSELA = 0x00;
    ANSELC = 0x00;
    ANSELD = 0x00;

    /*Globale Interrupt Konfiguration*/
    RCONbits.IPEN=1;
    INTCONbits.GIEL=1;
    INTCONbits.GIEH=1;

    /*Timer0 fr Tastenentprellung*/
    T0CON = 0b11000001;
    INTCON2bits.TMR0IP=0;
    INTCONbits.TMR0IE=1;

    /*Reset off Display*/
    CS1B = 1;
    RST = 1;
}

void initSPI(void)
{
    // SPI Mode 1,0
    SSP1CON1 = 0b00010010;  // Clock = FOSC/4 und CKP = 1
    SSP1STATbits.CKE = 0;   // CKE = 1
    SSP1CON1bits.SSPEN = 1; // SPI Modul ein
}

void configPWM(void)
{
  PR2=0xFF;
  CCPR1L=0x44;
  /*Half-Bridge - Single output PxA*/
  CCP1CONbits.CCP1M3 = 1;
  CCP1CONbits.CCP1M2 = 1;
  CCP1CONbits.CCP1M1 = 0;
  CCP1CONbits.CCP1M0 = 0;
  CCP1CONbits.P1M1 = 0;
  CCP1CONbits.P1M0 = 0;
  /*CCP1 using Timer2 in PWM Mode*/
  CCPTMRS0bits.C1TSEL1 = 0;
  CCPTMRS0bits.C1TSEL0 = 0;
  /*1:16 Postscale, Timer2 aus, Prescale=4*/
  T2CON=0x79;
  /*Timer2 an*/
  T2CONbits.TMR2ON=1;
}

/*
 * bertragung eines Bytes auf dem SPI Bus
 *
 * byte: Das zu bertragende Byte
 * Rckgabewert: Das empfangene Byte
 */
uint8_t sendSPI (uint8_t byte)
{   
    CS1B = 0;
    SSPBUF = byte;
    while(!SSP1STATbits.BF);
    CS1B = 1;
    return SSPBUF;
}