/* *****************************************************************************
 * File:        setup.c
 * Project:     Mini Anzeige Modul
 * Author:      Nicolas Meyertns
 * Version:     0.1.a.xc8
 * Web:         www.PIC-Projekte.de
 * ****************************************************************************/

#include <xc.h>
#include "setup.h"
#include "dogl.h"
#include "menu.h"

/*
 * berprfung ob und wenn ja, welche Taste gedrkt worden ist.
 *
 * *plastKeyDown:   Angabe des zuletzt gedrckten Tasters
 * Rckgabewert:    Der grdrckte Taster(kann auch NO_KEY sein).
 */
uint8_t keyPressed (uint8_t *plastKeyDown)
{
    uint8_t actKeyDown;

    if(EINGABE)
        actKeyDown = KEY_OK;
    else
        actKeyDown = NO_KEY;

    if( actKeyDown != *plastKeyDown)
    {
        *plastKeyDown = actKeyDown;
        return actKeyDown;
    }

    return NO_KEY;
}

/*
 * Die Funktion analysiert den Gray Code eines Drehgebers.
 * Da es Drehgeber in verschiedensten Ausfhren gibt ist beim Kauf
 * genaustens auf die Ausfhrung zu achten! Diese Routine ist fr
 * den Drehgeber STEC11B13 von Reichelt geschrieben!
 *
 * Verwendung:      Die Routine muss mit einem Timer Interrupt
 *                  aufgerufen werden. Stabil getestet beim Aufruf
 *                  alle 2ms. Eventuell geht es auch noch etwas
 *                  seltener, aber dass muss jeder selber testen!
 *
 * Rckgabewert:    Entweder KEY_UP bei Rechtsdrehung oder
 *                  KEY_UP bei Linksdrehung
 */
uint8_t encoder (void)
{
    switch(encState)
    {
        /*V00*/
        case 0x00:  if( PHI && !PHI90 )
                        encState = 0x10;
                    if( !PHI && PHI90 )
                        encState = 0x45;
                    break;
        /*V10*/
        case 0x10:  if( PHI &&  PHI90 )
                        encState = 0x11;
                    break;
        /*V11*/
        case 0x11:  if(!PHI &&  PHI90 )
                        encState = 0x01;
                    break;
        /*V01*/
        case 0x01:  if(!PHI && !PHI90 )
                    {
                        encState = 0x00;
                        return KEY_UP;
                    }
                    break;


        /*R01*/
        case 0x45:  if( PHI && PHI90 )
                        encState = 0x55;
                    break;
        /*R11*/
        case 0x55:  if( PHI && !PHI90 )
                        encState = 0x54;
                    break;
        /*R10*/
        case 0x54:  if(!PHI && !PHI90 )
                    {
                        encState = 0x44;
                        return KEY_DOWN;
                    }
                    break;
        /*R00*/
        case 0x44:  if(!PHI && PHI90 )
                        encState = 0x45;
                    if( PHI && !PHI90 )
                        encState = 0x10;
                    break;
    }
    return NO_KEY;
}

void mEnergy (void)
{
    drawBar("Helligkeit");
    while( !(input & KEY_OK) );
    input &= ~KEY_OK;
}

void mInfo (void)
{
    drawBar("Informationen");

    charChainDOGL(2, 0, NOINVERSE, "Firmware:");
    charChainDOGL(2,60, NOINVERSE, FW_VERSION);
    charChainDOGL(3, 0, NOINVERSE, "Entwickler:");
    charChainDOGL(3,60, NOINVERSE, "N. Meyertoens");
    charChainDOGL(4, 0, NOINVERSE, "Siehe auch:");
    charChainDOGL(4,60, NOINVERSE, "PIC-Projekte.de");

    while( !(input & KEY_OK) );
    input &= ~KEY_OK;
}